﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Windows.Forms;

using EllieMae.Encompass.Automation;
using EllieMae.Encompass.BusinessObjects.Loans;


namespace MacroExecPlugin
{
    public class EncompassPluginHandler
    {
        private volatile FieldMonitor_MacroExec field_monitor = null;

        public EncompassPluginHandler()
        {
            EncompassApplication.Login += new EventHandler(EncompassApplication_Login);
        }

        private void EncompassApplication_Login(object sender, EventArgs e)
        {
            EncompassApplication.LoanOpened += new EventHandler(Application_LoanOpened);
            EncompassApplication.LoanClosing += new EventHandler(Application_LoanClosing);
        }

        private void Application_LoanOpened(object sender, EventArgs e)
        {
            StartMonitor();
        }

        private void Application_LoanClosing(object sender, EventArgs e)
        {
            StopMonitor();
        }

        private void StartMonitor()
        {
            lock (this)
            {
                try
                {
                    StopMonitor();
                    Loan loan = EncompassApplication.CurrentLoan;
                    if (loan != null)
                    {
                        Utils2.AddToLog("Start Monitor");
                        field_monitor = new FieldMonitor_MacroExec(loan);
                        field_monitor.Start();
                    }
                }
                catch (Exception ex)
                {
                    Utils2.AddToLog("StartMonitor Exception: " + ex.Message);
                    Utils2.MsgBox_ErrorOK("StartMonitor Exception: " + ex.Message);
                }
            }
        }

        private void StopMonitor()
        {
            lock (this)
            {
                try
                {
                    if (field_monitor != null)
                    {
                        Utils2.AddToLog("Stop Monitor");
                        field_monitor.Stop();
                        field_monitor = null;
                    }
                }
                catch (Exception ex)
                {
                    Utils2.AddToLog("StopMonitor Exception: " + ex.Message);
                    Utils2.MsgBox_ErrorOK("StopMonitor Exception: " + ex.Message);
                }
            }
        }

    }
}

