﻿using System;
using System.Collections.Generic;
using System.Text;

using EllieMae.Encompass.BusinessObjects.Loans;

namespace MacroExecPlugin
{
    public class FieldMonitor_MacroExec
    {
        private Loan LOAN = null;

        // timer for delayed execution
        private volatile System.Timers.Timer timer_delayed_exec = null;

        // values we read from custom field trigger and need for delayed triggering
        private string TriggerFieldId = "";
        private string TriggerFieldValue = "";
        private int DelayMilliSec = 0;

        public FieldMonitor_MacroExec(Loan loan)
        {
            this.LOAN = loan;
        }

        public void Start()
        {
            LOAN.FieldChange += Loan_FieldChange;
        }

        public void Stop()
        {
            LOAN.FieldChange -= Loan_FieldChange;
            StopTimer_DelayedExec();
        }

        // don't throw
        private void Loan_FieldChange(object source, FieldChangeEventArgs e)
        {
            try
            {
                string sFieldID = e.FieldID;
                if (String.Equals(sFieldID, "CX.MACRO.ACTION", StringComparison.OrdinalIgnoreCase) ||
                    String.Equals(sFieldID, "CX.MACRO.SIGNATURE", StringComparison.OrdinalIgnoreCase))
                {
                    Utils2.AddToLog("FieldChange Start");

                    if (e.NewValue.Length > 0)
                    {
                        // 1. params
                        this.TriggerFieldId = e.FieldID;
                        this.TriggerFieldValue = e.NewValue;
                        this.DelayMilliSec = LOAN.Fields["CX.MACRO.DELAYMS"].ToInt();

                        // 2. clear fields
                        LOAN.Fields["CX.MACRO.ACTION"].Value = "";
                        LOAN.Fields["CX.MACRO.SIGNATURE"].Value = "";
                        LOAN.Fields["CX.MACRO.DELAYMS"].Value = "";

                        // 3. exec action or signature
                        HandleImmediateOrDelayedExec();
                    }

                    Utils2.AddToLog("FieldChange End");
                }
            }
            catch (Exception ex)
            {
                Utils2.AddToLog("FieldChange Exception: " + ex.Message);
            }
        }

        // throw on error
        private void HandleImmediateOrDelayedExec()
        {
            Utils2.AddToLog("Exec: Field=" + this.TriggerFieldId + " Value=" + this.TriggerFieldValue + " DelayMS=" + this.DelayMilliSec);
            if (this.DelayMilliSec < 1)
            {
                // immediate execution
                ExecuteNow();
            }
            else
            {
                // delayed execution
                StartTimer_DelayedExec();
            }
        }

        // throw on error
        private void StartTimer_DelayedExec()
        {
            lock (this)
            {
                // stop previous timer in case more than 1 delayed triggers
                StopTimer_DelayedExec(); 

                // delayed execution
                timer_delayed_exec = new System.Timers.Timer(this.DelayMilliSec);
                timer_delayed_exec.Elapsed += new System.Timers.ElapsedEventHandler(OnTimer_DelayedExec);
                timer_delayed_exec.AutoReset = false; // single-time
                // timer_delayed_exec.SynchronizingObject = frmMain; <- set this if there are cross-thread issues
                timer_delayed_exec.Enabled = true;
            }
        }

        // don't throw
        private void StopTimer_DelayedExec()
        {
            lock (this)
            {
                if (timer_delayed_exec != null)
                {
                    try
                    {
                        timer_delayed_exec.Enabled = false;
                        timer_delayed_exec.Dispose();
                    }
                    catch { } // ignore
                    timer_delayed_exec = null;
                }
            }
        }

        // don't throw
        private void OnTimer_DelayedExec(object sender, System.Timers.ElapsedEventArgs e)
        {
            lock (this)
            {
                try
                {
                    StopTimer_DelayedExec();
                    ExecuteNow();
                }
                catch (Exception ex)
                {
                    Utils2.AddToLog("DelayedTimer Exception: " + ex.Message);
                    Utils2.MsgBox_ErrorOK("DelayedTimer Exception: " + ex.Message);
                }
            }
        }

        // throw on error
        // Insnance Params: TriggerFieldId, TriggerFieldValue, DelayMilliSec
        private void ExecuteNow()
        {
            if (this.TriggerFieldId.IndexOf("ACTION", StringComparison.OrdinalIgnoreCase) >= 0)
            {
                // Exec Action
                Utils2.AddToLog("Exec Action: " + this.TriggerFieldValue);
                EllieMae.Encompass.Automation.Macro.ExecAction(this.TriggerFieldValue);
            }
            else if (this.TriggerFieldId.IndexOf("SIGNATURE", StringComparison.OrdinalIgnoreCase) >= 0)
            {
                // Exec Signature
                Utils2.AddToLog("Exec Signature: " + this.TriggerFieldValue);
                EllieMae.Encompass.Automation.Macro.ExecSignature(this.TriggerFieldValue);
            }
            else
            {
                throw new Exception ("Unexpected Exec field: " + this.TriggerFieldId);
            }
        }

    }
}
